



#include <allegro.h>

#include <cmath>

#define MAX_BLOOBS	64
#include "CBloob.h"


// timing variables
volatile int fps, frame_count, game_count;

// bitmaps
BITMAP *swap_screen, *background;

//	keeps track of frames each second
void fps_counter(void) {
	fps = frame_count;
	frame_count = 0;
}
END_OF_FUNCTION(fps_counter);

// keeps track of internal game speed
void game_counter(void) {
	game_count++;
}
END_OF_FUNCTION(game_counter);

// initiates the timers
void init_timers() {
	install_timer();
	LOCK_VARIABLE(game_count);
	LOCK_VARIABLE(frame_count);
	LOCK_VARIABLE(fps);
	LOCK_VARIABLE(frame_count);
	LOCK_FUNCTION(fps_counter);
	install_int(fps_counter,1000);
	fps=0;
	frame_count=0;
	LOCK_FUNCTION(game_counter);
	install_int(game_counter, 10);
}


int main(int argc, char **argv) {
	int i;
	float tick = 0;

	allegro_init();
	set_color_depth(8);
	set_gfx_mode(GFX_AUTODETECT_WINDOWED, 640, 480, 0, 0);
	install_keyboard();
	init_timers();
	text_mode(-1);

	swap_screen = create_bitmap(320, 240);
	background = create_bitmap(320, 240);

	CBloob *b[MAX_BLOOBS];
	RGB rgb = { 0, 0, 0 };

	clear(screen);
	set_color(0, &rgb);

	// setup palette
	for(i=0;i<128;i++) {
		rgb.b = (i*63)/128;
		rgb.g = (i>=64 ? (i-64) : 0);
		rgb.r = (i>=96 ? (i-96)*2 : 0);
		set_color(i, &rgb);
	}

	// create bloobs
	for(i=0;i<MAX_BLOOBS;i++) {
		int size = MAX(MIN(MAX_BLOOBS - i / 2, 64), 5);
		b[i] = new CBloob(size, 0, 127);
	}

	// set up bloob blender
	b[0]->setupBlender();

	// create background
	clear(background);
	// color
	for(int x = 0; x < 320; x ++) {
		for(int y = 0; y < 240; y ++) {
			float power = (60 - x) * (60 - x) + (40 - y) * (40 - y);
			putpixel(background, x, y, (int)(MAX(48 - power/700, 0))%128);
		}
	}


	// loop
	int w = 48, h = 32;
	while(!key[KEY_ESC]) {
		game_count = 0;
		frame_count ++;
		tick += 0.02f;

		// move blobs
		for(i=0;i<MAX_BLOOBS;i++) {
			b[i]->x = 160 - b[i]->bmp->w / 2 + w * cos(tick + i * 3) + w * cos(tick/3 - i * 7);
			b[i]->y = 120 - b[i]->bmp->h / 2 + h * sin(tick/9 - i * 5) + h * sin(tick + i * 3);
		}

		// draw
		// clear(swap_screen);
		blit(background, swap_screen, 0, 0, 0, 0, 320, 240);

		// draw blobs
		for(i=0;i<MAX_BLOOBS;i++) {
			b[i]->draw(swap_screen);
		}

		// show fps
		// textprintf(swap_screen, font, 1, 1, makecol(100, 100, 100), "%d", fps);

		// blit to screen
		if (!key[KEY_TAB]) vsync();
		stretch_blit(swap_screen, screen, 0, 0, 320, 240, 0, 0, 640, 480);

		// old fashined timing device
		if (!key[KEY_TAB]) while(!game_count);
	}

	// delete blobs
	for(i=0;i<MAX_BLOOBS;i++) {
		delete b[i];
	}


	return 0;
}
END_OF_MAIN()



